/*

s_A078760.c	s_A209936.c	s_A080577.c
Copyright (C) March, 2012 Sergei Viznyuk <phystech@hotmail.com>

Print OEIS sequences A078760 A209936 A080577

This program takes positive integers n and k as inputs.
For the given value of n it prints out all partitions of n into <=k parts.
The first row contains the header M,D\k and ordered values of 1,2,..,k
The subsequent rows contain comma-separated values M,D,m1,m2,..,mk
, where
	M  - value of multinomial coefficient M=n!/(m1!m2!..mk!)
	     where m1+m2+..+mk=n is the given partition of n into <=k parts
	D  - multiplicity of the given m1,m2,..,mk partition, i.e. the number
	     of ways m1,m2,..,mk values can be distributed between k bins
	m1 - population of first partition
	m2 - population of second partition
	*  - ..
	*  - ..
	mk - population of k-th partition

The values of M by row make up http://oeis.org/A078760 sequence.
The values of D by row make up http://oeis.org/A209936 sequence.
The values m1,m2,..,mk (omitting trailing zero values) by row make up A080577
sequence when input parameter k=n.

The algorithm for finding partitions of a given integer n is implemented
from the following paper:
Constant Time Generation of Integer Partitions
YAMANAKA et al. IEICE Trans Fundamentals.2007; E90-A: 888-895 
The example MATLAB code used:
http://www.mathworks.com/matlabcentral/newsreader/view_thread/162379

This code is limited by LLONG_MAX values. If arguments n and k result in
values of M exceeding LLONG_MAX the code returns overflow error.
The code is performing checksum on computed sequence by verifying sum(M*D)=k^n

This is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#ifdef __CYGWIN__
#define EXP exp
#define LGAMMA lgamma
#else
#define EXP expl
#define LGAMMA lgammal
#endif

typedef struct	{ long long unsigned M,D; long unsigned k,m[]; } wt;

static wt **s;
static long unsigned k,k0,cr=0,inc=1000;
static long double dn,en;

static long long unsigned powi(long long unsigned b, long unsigned p)
{ if( p > 1 ) return b*powi(b,p-1);
  if ( !p ) return 0;
  return b;
}

static void cmpt(wt *a)
{ long unsigned i=0,j=0,k=a->k,*m=a->m;
  long double d=dn, e=en;
  while ( i<k )
    { if ( m[i] > 1 ) d-=LGAMMA(1.0L+m[i]);
      if ( m[i]!=m[j] ) { if ( i>j+1 ) e-=LGAMMA(1.0L+i-j); j=i; }
      i++;
    }
  a->M=llround(EXP(d));
  if ( i>j+1 ) e-=LGAMMA(1.0L+i-j);
  if ( k0>i+1 ) e-=LGAMMA(1.0L+k0-i);
  a->D=llround(EXP(e));
}

static int compar(wt **ap,wt **bp)
{ wt *a=*ap, *b=*bp;
  long unsigned *c1=a->m,*c2=b->m;
  long unsigned i=0,k=a->k;

  if ( !a->M ) cmpt(a);
  if ( !b->M ) cmpt(b);
  while ( i < k )
    if ( c1[i] < c2[i] ) return 1;
    else if ( c1[i] > c2[i] ) return -1;
    else i++;
  return 0;
}

static void children3(wt *a)
{ wt *b;
  long unsigned i=a->k,*m=a->m;
  if ( cr==inc )
    s=realloc(s,(inc+=inc)*sizeof(wt *));
  s[cr++]=a;
  if ( *m > m[1] )
    { if ( i < k )
	{ b=malloc(sizeof(wt)+(i+1)*sizeof(long unsigned));
	  memcpy(b,a,sizeof(wt)+i*sizeof(long unsigned));
	  (*b->m)--; b->m[i]=1; b->k++; children3(b);
	}
      if ( m[i-2] > m[i-1] && ( i > 2 || (m[i-2]-m[i-1]) > 1 ) )
	{ b=malloc(sizeof(wt)+i*sizeof(long unsigned));
	  memcpy(b,a,sizeof(wt)+i*sizeof(long unsigned));
	  (*b->m)--; b->m[i-1]++; children3(b);
	}
    }
}

int main(int argn,char *argc[])
{ wt *a;
  long unsigned i,j,n,h,*m;
  long long unsigned p,q;
  if ( argn!=3 )
    { fprintf(stderr,"Usage: %s <n> <k>\n",argc[0]); return -1; }
  n=atol(argc[1]);
  k=k0=atol(argc[2]);
  if ( n < k ) k=n;
  if ( !k ) return 0;
  fprintf(stdout,"M,D\\k"); i=0;
  while ( i<k0 ) fprintf(stdout,",%lu",++i); puts("");
  fprintf(stdout,"1,%lu,%lu",k0,n);
  for (i=1;i<k0;i++) fprintf(stdout,",0"); puts("");
  if ( k==1 ) return 0; q=k0;
  dn=LGAMMA(1.0L+n);
  en=LGAMMA(1.0L+k0);
  s=malloc(inc*sizeof(long unsigned *));
  a=malloc(sizeof(wt)+2*sizeof(long unsigned));
  a->M=0; a->k=2; *a->m=n-1; a->m[1]=1;
  children3(a);
  if ( cr > 1 )
    qsort(s,cr,sizeof(long unsigned *),
    (int (*)(const void *,const void *))&compar);
  else
    compar(&a,&a);
  for (i=0;i<cr;i++)
    { a=s[i]; m=a->m; h=a->k; j=0;
      fprintf(stdout,"%llu,%llu",a->M,a->D); q+=a->M*a->D;
      for (j=0;j<h;j++) fprintf(stdout,",%lu",m[j]);
      while ( j++<k0 ) fprintf(stdout,",0");
      puts(""); free(a);
    }
  free(s);
  p=powi(k0,n);
  if ( q==p ) return 0;
  fprintf(stderr,
  "*** Overflow error: k^n=%llu\n" 
  "               sum(M*D)=%llu\n",p,q);
  return 1;
}
